@@ -1,4 +1,4 @@
-from core import Locust, TaskSet, WebLocust, SubLocust, task
+from core import HttpLocust, Locust, TaskSet, task
 from exception import InterruptTaskSet, ResponseError, RescheduleTaskImmediately
 
-version = "0.6.2"
+version = "0.7.0"
