@@ -52,6 +52,15 @@ def task(weight=1):
         return decorator_func
 
 
+class NoClientWarningRaiser(object):
+    """
+    The purpose of this class is to emit a sensible error message for old test scripts that 
+    inherits from Locust, and expects there to be an HTTP client under the client attribute.
+    """
+    def __getattr__(self, _):
+        raise LocustError("No client instantiated. Did you intend to inherit from HttpLocust?")
+
+
 class Locust(object):
     """
     Represents a "user" which is to be hatched and attack the system that is to be load tested.
@@ -59,8 +68,9 @@ class Locust(object):
     The behaviour of this user is defined by the task_set attribute, which should point to a 
     :py:class:`TaskSet <locust.core.TaskSet>` class.
     
-    This class creates a *client* attribute on instantiation which is an HTTP client with support 
-    for keeping a user session between requests.
+    This class should usually be subclassed by a class that defines some kind of client. For 
+    example when load testing an HTTP system, you probably want to use the 
+    :py:class:`HttpLocust <locust.core.HttpLocust>` class.
     """
     
     host = None
@@ -72,15 +82,6 @@ class Locust(object):
     max_wait = 1000
     """Maximum waiting time between the execution of locust tasks"""
     
-    avg_wait = None
-    """Average waiting time wanted between the execution of locust tasks"""
-    
-    client = None
-    """
-    Instance of HttpSession that is created upon instantiation of Locust. 
-    The client support cookies, and therefore keeps the session between HTTP requests.
-    """
-    
     task_set = None
     """TaskSet class that defines the execution behaviour of this locust"""
     
@@ -89,23 +90,47 @@ class Locust(object):
 
     weight = 10
     """Probability of locust being chosen. The higher the weight, the greater is the chance of it being chosen."""
+        
+    client = NoClientWarningRaiser()
+    _catch_exceptions = False
     
     def __init__(self):
         super(Locust, self).__init__()
-        if self.host is None:
-            raise LocustError("You must specify the base host. Either in the host attribute in the Locust class, or on the command line using the --host option.")
-        
-        self.client = HttpSession(base_url=self.host)
     
     def run(self):
         try:
             self.task_set(self).run()
         except StopLocust:
             pass
-        except (RescheduleTask, RescheduleTaskImmediately), e:
+        except (RescheduleTask, RescheduleTaskImmediately) as e:
             raise LocustError, LocustError("A task inside a Locust class' main TaskSet (`%s.task_set` of type `%s`) seems to have called interrupt() or raised an InterruptTaskSet exception. The interrupt() function is used to hand over execution to a parent TaskSet, and should never be called in the main TaskSet which a Locust class' task_set attribute points to." % (type(self).__name__, self.task_set.__name__)), sys.exc_info()[2]
 
 
+class HttpLocust(Locust):
+    """
+    Represents an HTTP "user" which is to be hatched and attack the system that is to be load tested.
+    
+    The behaviour of this user is defined by the task_set attribute, which should point to a 
+    :py:class:`TaskSet <locust.core.TaskSet>` class.
+    
+    This class creates a *client* attribute on instantiation which is an HTTP client with support 
+    for keeping a user session between requests.
+    """
+    
+    client = None
+    """
+    Instance of HttpSession that is created upon instantiation of Locust. 
+    The client support cookies, and therefore keeps the session between HTTP requests.
+    """
+    
+    def __init__(self):
+        super(HttpLocust, self).__init__()
+        if self.host is None:
+            raise LocustError("You must specify the base host. Either in the host attribute in the Locust class, or on the command line using the --host option.")
+        
+        self.client = HttpSession(base_url=self.host)
+
+
 class TaskSetMeta(type):
     """
     Meta class for the main Locust class. It's used to allow Locust classes to specify task execution 
@@ -187,15 +212,6 @@ class TaskSet(object):
     TaskSet.
     """
     
-    avg_wait = None
-    """Average waiting time wanted between the execution of locust tasks"""
-    
-    client = None
-    """
-    Reference to the :py:attr:`client <locust.core.Locust.client>` attribute of the root 
-    Locust instance.
-    """
-    
     locust = None
     """Will refer to the root Locust class instance when the TaskSet has been instantiated"""
 
@@ -208,8 +224,6 @@ class TaskSet(object):
     __metaclass__ = TaskSetMeta    
     
     def __init__(self, parent):
-        self._avg_wait = 0
-        self._avg_wait_ctr = 0
         self._task_queue = []
         self._time_start = time()
         
@@ -221,22 +235,26 @@ class TaskSet(object):
             raise LocustError("TaskSet should be called with Locust instance or TaskSet instance as first argument")
 
         self.parent = parent
-        self.client = self.locust.client
         
-        # if this class doesn't have a min_wait, max_wait or avg_wait defined, copy it from Locust
+        # if this class doesn't have a min_wait or max_wait defined, copy it from Locust
         if not self.min_wait:
             self.min_wait = self.locust.min_wait
         if not self.max_wait:
             self.max_wait = self.locust.max_wait
-        if not self.avg_wait:
-            self.avg_wait = self.locust.avg_wait
 
     def run(self, *args, **kwargs):
         self.args = args
         self.kwargs = kwargs
         
-        if hasattr(self, "on_start"):
-            self.on_start()
+        try:
+            if hasattr(self, "on_start"):
+                self.on_start()
+        except InterruptTaskSet as e:
+            if e.reschedule:
+                raise RescheduleTaskImmediately, e, sys.exc_info()[2]
+            else:
+                raise RescheduleTask, e, sys.exc_info()[2]
+        
         while (True):
             try:
                 if self.locust.stop_timeout is not None and time() - self._time_start > self.locust.stop_timeout:
@@ -253,7 +271,7 @@ class TaskSet(object):
                     self.wait()
                 else:
                     self.wait()
-            except InterruptTaskSet, e:
+            except InterruptTaskSet as e:
                 if e.reschedule:
                     raise RescheduleTaskImmediately, e, sys.exc_info()[2]
                 else:
@@ -262,10 +280,13 @@ class TaskSet(object):
                 raise
             except GreenletExit:
                 raise
-            except Exception, e:
-                events.locust_error.fire(self, e, sys.exc_info()[2])
-                sys.stderr.write("\n" + traceback.format_exc())
-                self.wait()
+            except Exception as e:
+                events.locust_error.fire(locust_instance=self, exception=e, tb=sys.exc_info()[2])
+                if self.locust._catch_exceptions:
+                    sys.stderr.write("\n" + traceback.format_exc())
+                    self.wait()
+                else:
+                    raise
     
     def execute_next_task(self):
         task = self._task_queue.pop(0)
@@ -273,7 +294,7 @@ class TaskSet(object):
     
     def execute_task(self, task, *args, **kwargs):
         # check if the function is a method bound to the current locust, and if so, don't pass self as first argument
-        if hasattr(task, "im_self") and task.im_self == self:
+        if hasattr(task, "im_self") and task.__self__ == self:
             # task is a bound method on self
             task(*args, **kwargs)
         elif hasattr(task, "tasks") and issubclass(task, TaskSet):
@@ -304,26 +325,7 @@ class TaskSet(object):
         return random.choice(self.tasks)
     
     def wait(self):
-        if self.avg_wait:
-            # Handle (strive for) average wait time
-            if self._avg_wait:
-                if self._avg_wait >= self.avg_wait:
-                    # Want shorter wait
-                    millis = random.randint(self.min_wait, self.avg_wait)
-                else:
-                    # Want longer wait
-                    millis = random.randint(self.avg_wait, self.max_wait)
-                self._avg_wait = ((self._avg_wait * self._avg_wait_ctr) + millis) / (self._avg_wait_ctr + 1.0)
-            else:
-                # Average specified but is first run
-                radius = min(self.avg_wait - self.min_wait, self.max_wait - self.avg_wait)
-                millis = random.randint(self.avg_wait - radius, self.avg_wait + radius)
-                self._avg_wait = millis
-            self._avg_wait_ctr += 1
-        else:
-            # Ignore average wait
-            millis = random.randint(self.min_wait, self.max_wait)
-
+        millis = random.randint(self.min_wait, self.max_wait)
         seconds = millis / 1000.0
         self._sleep(seconds)
 
@@ -342,20 +344,12 @@ class TaskSet(object):
         classes further down the hierarchy.
         """
         raise InterruptTaskSet(reschedule)
+    
+    @property
+    def client(self):
+        """
+        Reference to the :py:attr:`client <locust.core.Locust.client>` attribute of the root 
+        Locust instance.
+        """
+        return self.locust.client
 
-class WebLocust(Locust):
-    def __init__(self, *args, **kwargs):
-        warnings.warn("WebLocust class has been, deprecated. Use Locust class instead.")
-        super(WebLocust, self).__init__()
-
-
-class SubLocust(object):
-    """
-    Class for making a sub Locust that can be included as a task inside of a normal Locust/WebLocus,
-    as well as inside another sub locust.
-
-    When the parent locust enters the sub locust, it will not
-    continue executing it's tasks until a task in the sub locust has called the interrupt() function.
-    """
-    def __init__(self, *args, **kwargs):
-        raise DeprecationWarning("The SubLocust class has been deprecated. Use TaskSet classes instead.")
