@@ -7,17 +7,18 @@ import os
 import signal
 import inspect
 import logging
+import socket
 from optparse import OptionParser
 
 import web
 from log import setup_logging, console_logger
 from stats import stats_printer, print_percentile_stats, print_error_report, print_stats
 from inspectlocust import print_task_ratio, get_task_ratio_dict
-from core import Locust, WebLocust
+from core import Locust, HttpLocust
 from runners import MasterLocustRunner, SlaveLocustRunner, LocalLocustRunner
 import events
 
-_internals = [Locust, WebLocust]
+_internals = [Locust, HttpLocust]
 version = locust.version
 
 def parse_options():
@@ -37,6 +38,21 @@ def parse_options():
         help="Host to load test in the following format: http://10.21.32.33"
     )
 
+    parser.add_option(
+        '--web-host',
+        dest="web_host",
+        default="",
+        help="Host to bind the web interface to. Defaults to '' (all interfaces)"
+    )
+    
+    parser.add_option(
+        '-P', '--port', '--web-port',
+        type="int",
+        dest="port",
+        default=8089,
+        help="Port on which to run web host"
+    )
+    
     parser.add_option(
         '-f', '--locustfile',
         dest='locustfile',
@@ -71,6 +87,33 @@ def parse_options():
         default="127.0.0.1",
         help="Host or IP address of locust master for distributed load testing. Only used when running with --slave. Defaults to 127.0.0.1."
     )
+    
+    parser.add_option(
+        '--master-port',
+        action='store',
+        type='int',
+        dest='master_port',
+        default=5557,
+        help="The port to connect to that is used by the locust master for distributed load testing. Only used when running with --slave. Defaults to 5557. Note that slaves will also connect to the master node on this port + 1."
+    )
+
+    parser.add_option(
+        '--master-bind-host',
+        action='store',
+        type='str',
+        dest='master_bind_host',
+        default="*",
+        help="Interfaces (hostname, ip) that locust master should bind to. Only used when running with --master. Defaults to * (all available interfaces)."
+    )
+    
+    parser.add_option(
+        '--master-bind-port',
+        action='store',
+        type='int',
+        dest='master_bind_port',
+        default=5557,
+        help="Port that locust master should bind to. Only used when running with --master. Defaults to 5557. Note that Locust will also use this port + 1, so by default the master node will bind to 5557 and 5558."
+    )
 
     # if we should print stats in the console
     parser.add_option(
@@ -130,15 +173,6 @@ def parse_options():
         default=None,
         help="Path to log file. If not set, log will go to stdout/stderr",
     )
-
-    # ramp feature enabled option
-    parser.add_option(
-        '--ramp',
-        action='store_true',
-        dest='ramp',
-        default=False,
-        help="Enables the auto tuning ramping feature for finding highest stable client count. NOTE having ramp enabled will add some more overhead for additional stats gathering"
-    )
     
     # if we should print stats in the console
     parser.add_option(
@@ -148,6 +182,15 @@ def parse_options():
         default=False,
         help="Print stats in the console"
     )
+
+    # only print summary stats
+    parser.add_option(
+       '--only-summary',
+       action='store_true',
+       dest='only_summary',
+       default=False,
+       help='Only print the summary stats'
+    )
     
     # List locust commands found in loaded locust files/source files
     parser.add_option(
@@ -238,10 +281,11 @@ def is_locust(tup):
     Takes (name, object) tuple, returns True if it's a public Locust subclass.
     """
     name, item = tup
-    return (
+    return bool(
         inspect.isclass(item)
         and issubclass(item, Locust)
-        and (item not in _internals)
+        and hasattr(item, "task_set")
+        and getattr(item, "task_set")
         and not name.startswith('_')
     )
 
@@ -350,22 +394,26 @@ def main():
 
     if not options.no_web and not options.slave:
         # spawn web greenlet
-        logger.info("Starting web monitor on port 8089")
-        main_greenlet = gevent.spawn(web.start, locust_classes, options.hatch_rate, options.num_clients, options.num_requests, options.ramp)
+        logger.info("Starting web monitor at %s:%s" % (options.web_host or "*", options.port))
+        main_greenlet = gevent.spawn(web.start, locust_classes, options)
     
     if not options.master and not options.slave:
-        runners.locust_runner = LocalLocustRunner(locust_classes, options.hatch_rate, options.num_clients, options.num_requests, options.host)
+        runners.locust_runner = LocalLocustRunner(locust_classes, options)
         # spawn client spawning/hatching greenlet
         if options.no_web:
             runners.locust_runner.start_hatching(wait=True)
             main_greenlet = runners.locust_runner.greenlet
     elif options.master:
-        runners.locust_runner = MasterLocustRunner(locust_classes, options.hatch_rate, options.num_clients, num_requests=options.num_requests, host=options.host, master_host=options.master_host)
+        runners.locust_runner = MasterLocustRunner(locust_classes, options)
     elif options.slave:
-        runners.locust_runner = SlaveLocustRunner(locust_classes, options.hatch_rate, options.num_clients, num_requests=options.num_requests, host=options.host, master_host=options.master_host)
-        main_greenlet = runners.locust_runner.greenlet
+        try:
+            runners.locust_runner = SlaveLocustRunner(locust_classes, options)
+            main_greenlet = runners.locust_runner.greenlet
+        except socket.error, e:
+            logger.error("Failed to connect to the Locust master: %s", e)
+            sys.exit(-1)
     
-    if options.print_stats or (options.no_web and not options.slave):
+    if not options.only_summary and (options.print_stats or (options.no_web and not options.slave)):
         # spawn stats printing greenlet
         gevent.spawn(stats_printer)
     
@@ -374,9 +422,11 @@ def main():
         Shut down locust by firing quitting event, printing stats and exiting
         """
         logger.info("Shutting down, bye..")
+
         events.quitting.fire()
         print_stats(runners.locust_runner.request_stats)
         print_percentile_stats(runners.locust_runner.request_stats)
+
         print_error_report()
         sys.exit(code)
     
@@ -389,7 +439,8 @@ def main():
     try:
         logger.info("Starting Locust %s" % version)
         main_greenlet.join()
-    except KeyboardInterrupt, e:
+        shutdown(0)
+    except KeyboardInterrupt as e:
         shutdown(0)
 
 if __name__ == '__main__':
