@@ -1,12 +1,12 @@
 import unittest
 
-from locust.core import Locust, TaskSet, task, events
+from locust.core import HttpLocust, Locust, TaskSet, task, events
 from locust import ResponseError, InterruptTaskSet
 from locust.exception import CatchResponseError, RescheduleTask, RescheduleTaskImmediately, LocustError
 
-from testcases import WebserverTestCase
+from testcases import LocustTestCase, WebserverTestCase
 
-class TestTaskSet(unittest.TestCase):
+class TestTaskSet(LocustTestCase):
     def setUp(self):
         super(TestTaskSet, self).setUp()
         
@@ -151,7 +151,7 @@ class TestTaskSet(unittest.TestCase):
         
         l = MySubTaskSet(self.locust)
         self.assertEqual(2, len(l.tasks))
-        self.assertEqual([t1, MySubTaskSet.t2.im_func], l.tasks)
+        self.assertEqual([t1, MySubTaskSet.t2.__func__], l.tasks)
     
     def test_task_decorator_with_or_without_argument(self):
         class MyTaskSet(TaskSet):
@@ -257,7 +257,7 @@ class TestTaskSet(unittest.TestCase):
         
         try:
             l.run()
-        except LocustError, e:
+        except LocustError as e:
             self.assertTrue("MyLocust" in e.args[0], "MyLocust should have been referred to in the exception message")
             self.assertTrue("MyTaskSet" in e.args[0], "MyTaskSet should have been referred to in the exception message")
         except:
@@ -265,12 +265,30 @@ class TestTaskSet(unittest.TestCase):
         
         try:
             l2.run()
-        except LocustError, e:
+        except LocustError as e:
             self.assertTrue("MyLocust2" in e.args[0], "MyLocust2 should have been referred to in the exception message")
             self.assertTrue("MyTaskSet2" in e.args[0], "MyTaskSet2 should have been referred to in the exception message")
         except:
             raise
     
+    def test_on_start_interrupt(self):
+        class SubTaskSet(TaskSet):
+            def on_start(self):
+                if self.kwargs["reschedule"]:
+                    self.interrupt(reschedule=True)
+                else:
+                    self.interrupt(reschedule=False)
+        
+        class MyLocust(Locust):
+            host = ""
+            task_set = SubTaskSet
+        
+        l = MyLocust()
+        task_set = SubTaskSet(l)
+        self.assertRaises(RescheduleTaskImmediately, lambda: task_set.run(reschedule=True))
+        self.assertRaises(RescheduleTask, lambda: task_set.run(reschedule=False))
+
+    
     def test_parent_attribute(self):
         from locust.exception import StopLocust
         parents = {}
@@ -304,7 +322,7 @@ class TestWebLocustClass(WebserverTestCase):
         self.response = ""
         def t1(l):
             self.response = l.client.get("/ultra_fast")
-        class MyLocust(Locust):
+        class MyLocust(HttpLocust):
             tasks = [t1]
             host = "http://127.0.0.1:%i" % self.port
 
@@ -313,28 +331,28 @@ class TestWebLocustClass(WebserverTestCase):
         self.assertEqual(self.response.content, "This is an ultra fast response")
 
     def test_client_request_headers(self):
-        class MyLocust(Locust):
+        class MyLocust(HttpLocust):
             host = "http://127.0.0.1:%i" % self.port
 
         locust = MyLocust()
         self.assertEqual("hello", locust.client.get("/request_header_test", headers={"X-Header-Test":"hello"}).content)
 
     def test_client_get(self):
-        class MyLocust(Locust):
+        class MyLocust(HttpLocust):
             host = "http://127.0.0.1:%i" % self.port
 
         locust = MyLocust()
         self.assertEqual("GET", locust.client.get("/request_method").content)
     
     def test_client_get_absolute_url(self):
-        class MyLocust(Locust):
+        class MyLocust(HttpLocust):
             host = "http://127.0.0.1:%i" % self.port
 
         locust = MyLocust()
         self.assertEqual("GET", locust.client.get("http://127.0.0.1:%i/request_method" % self.port).content)
 
     def test_client_post(self):
-        class MyLocust(Locust):
+        class MyLocust(HttpLocust):
             host = "http://127.0.0.1:%i" % self.port
 
         locust = MyLocust()
@@ -342,7 +360,7 @@ class TestWebLocustClass(WebserverTestCase):
         self.assertEqual("hello world", locust.client.post("/post", {"arg":"hello world"}).content)
 
     def test_client_put(self):
-        class MyLocust(Locust):
+        class MyLocust(HttpLocust):
             host = "http://127.0.0.1:%i" % self.port
 
         locust = MyLocust()
@@ -350,7 +368,7 @@ class TestWebLocustClass(WebserverTestCase):
         self.assertEqual("hello world", locust.client.put("/put", {"arg":"hello world"}).content)
 
     def test_client_delete(self):
-        class MyLocust(Locust):
+        class MyLocust(HttpLocust):
             host = "http://127.0.0.1:%i" % self.port
 
         locust = MyLocust()
@@ -358,20 +376,20 @@ class TestWebLocustClass(WebserverTestCase):
         self.assertEqual(200, locust.client.delete("/request_method").status_code)
 
     def test_client_head(self):
-        class MyLocust(Locust):
+        class MyLocust(HttpLocust):
             host = "http://127.0.0.1:%i" % self.port
 
         locust = MyLocust()
         self.assertEqual(200, locust.client.head("/request_method").status_code)
 
     def test_client_basic_auth(self):
-        class MyLocust(Locust):
+        class MyLocust(HttpLocust):
             host = "http://127.0.0.1:%i" % self.port
 
-        class MyAuthorizedLocust(Locust):
+        class MyAuthorizedLocust(HttpLocust):
             host = "http://locust:menace@127.0.0.1:%i" % self.port
 
-        class MyUnauthorizedLocust(Locust):
+        class MyUnauthorizedLocust(HttpLocust):
             host = "http://locust:wrong@127.0.0.1:%i" % self.port
 
         locust = MyLocust()
@@ -382,10 +400,10 @@ class TestWebLocustClass(WebserverTestCase):
         self.assertFalse(unauthorized.client.get("/basic_auth"))
     
     def test_log_request_name_argument(self):
-        from locust.stats import RequestStats
+        from locust.stats import RequestStats, global_stats
         self.response = ""
         
-        class MyLocust(Locust):
+        class MyLocust(HttpLocust):
             tasks = []
             host = "http://127.0.0.1:%i" % self.port
             
@@ -396,25 +414,41 @@ class TestWebLocustClass(WebserverTestCase):
         my_locust = MyLocust()
         my_locust.t1()
         
-        self.assertEqual(1, RequestStats.get("GET", "new name!").num_reqs)
-        self.assertEqual(0, RequestStats.get("GET", "/ultra_fast").num_reqs)
+        self.assertEqual(1, global_stats.get("new name!", "GET").num_requests)
+        self.assertEqual(0, global_stats.get("/ultra_fast", "GET").num_requests)
+    
+    def test_locust_client_error(self):
+        class MyTaskSet(TaskSet):
+            @task
+            def t1(self):
+                self.client.get("/")
+                self.interrupt()
+        
+        class MyLocust(Locust):
+            host = "http://127.0.0.1:%i" % self.port
+            task_set = MyTaskSet
+        
+        my_locust = MyLocust()
+        self.assertRaises(LocustError, lambda: my_locust.client.get("/"))
+        my_taskset = MyTaskSet(my_locust)
+        self.assertRaises(LocustError, lambda: my_taskset.client.get("/"))
 
 
 class TestCatchResponse(WebserverTestCase):
     def setUp(self):
         super(TestCatchResponse, self).setUp()
         
-        class MyLocust(Locust):
+        class MyLocust(HttpLocust):
             host = "http://127.0.0.1:%i" % self.port
 
         self.locust = MyLocust()
         
         self.num_failures = 0
         self.num_success = 0
-        def on_failure(method, path, response_time, exception, response):
+        def on_failure(request_type, name, response_time, exception):
             self.num_failures += 1
             self.last_failure_exception = exception
-        def on_success(a, b, c, d):
+        def on_success(**kwargs):
             self.num_success += 1
         events.request_failure += on_failure
         events.request_success += on_success
@@ -469,7 +503,7 @@ class TestCatchResponse(WebserverTestCase):
             def interrupted_task(self):
                 with self.client.get("/ultra_fast", catch_response=True) as r:
                     raise InterruptTaskSet()
-        class MyLocust(Locust):
+        class MyLocust(HttpLocust):
             host = "http://127.0.0.1:%i" % self.port
             task_set = MyTaskSet
         
@@ -480,22 +514,22 @@ class TestCatchResponse(WebserverTestCase):
         self.assertEqual(0, self.num_success)
     
     def test_catch_response_connection_error_success(self):
-        class MyLocust(Locust):
+        class MyLocust(HttpLocust):
             host = "http://127.0.0.1:1"
         l = MyLocust()
         with l.client.get("/", catch_response=True) as r:
-            self.assertFalse(r)
+            self.assertEqual(r.status_code, 0)
             self.assertEqual(None, r.content)
             r.success()
         self.assertEqual(1, self.num_success)
         self.assertEqual(0, self.num_failures)
     
     def test_catch_response_connection_error_fail(self):
-        class MyLocust(Locust):
+        class MyLocust(HttpLocust):
             host = "http://127.0.0.1:1"
         l = MyLocust()
         with l.client.get("/", catch_response=True) as r:
-            self.assertFalse(r)
+            self.assertEqual(r.status_code, 0)
             self.assertEqual(None, r.content)
             r.success()
         self.assertEqual(1, self.num_success)
